#include "settingsdialog.h"
#include "ui_settingsdialog.h"

void maemoBanner(QString string)
{
    QString program = "dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:\"" + string + "\"";
    system(program.toStdString().c_str());
}

SettingsDialog::SettingsDialog(QOrganizerManager* manager, QWidget *parent) :
    QDialog(parent), defaultManager(manager), ui(new Ui::SettingsDialog)
{
    ui->setupUi(this);

    readSettings();

    createCalendarsCheckboxes();
}

SettingsDialog::~SettingsDialog()
{
    delete ui;
}

void SettingsDialog::readSettings()
{
    int fontSize;
    int numberOfRows;
    int rowWidth;
    int rowHeight;
    bool autoResize;
    int alphaValue;
    int widgetCornersRoundness;


    QSettings settings;
    numberOfRows = settings.value("numberOfRows", 7).toInt();
    fontSize = settings.value("fontSize", 15).toInt();
    rowHeight = settings.value("rowHeight", 40).toInt();
    rowWidth = settings.value("rowWidth", 400).toInt();
    autoResize = settings.value("autoResize", false).toBool();
    alphaValue = settings.value("alphaValue", 60).toInt();
    widgetCornersRoundness = settings.value("widgetCornersRoundness", 8).toInt();
    collectionIdsList = settings.value("collectionIdsList", QStringList()).toStringList();

    ui->rowHeightSpinBox->setValue(rowHeight);
    ui->alphaValueSlider->setValue(alphaValue);
    ui->rowWidthSpinBox->setValue(rowWidth);
    ui->numberOfRowsSpinBox->setValue(numberOfRows);
    ui->fontSizeSpinBox->setValue(fontSize);
    ui->widgetCornersRoundnessSpinBox->setValue(widgetCornersRoundness);

}

void SettingsDialog::saveSettings()
{
    QSettings settings;
    settings.setValue("numberOfRows", ui->numberOfRowsSpinBox->value());
    settings.setValue("rowWidth", ui->rowWidthSpinBox->value());
    settings.setValue("rowHeight", ui->rowHeightSpinBox->value());
    settings.setValue("alphaValue", ui->alphaValueSlider->value());
    settings.setValue("fontSize", ui->fontSizeSpinBox->value());
    settings.setValue("widgetCornersRoundness", ui->widgetCornersRoundnessSpinBox->value());

    QStringList collectionIdsList;

    for (int i = 0; i < calendarsCheckBoxList.size(); i++)
    {
        if ( calendarsCheckBoxList.at(i)->isChecked() )
        {
            QString id = calendarsCheckBoxList.at(i)->property("id").toString();
            collectionIdsList.append( id );
        }
    }

    settings.setValue("collectionIdsList", collectionIdsList);
}

void SettingsDialog::on_saveButton_clicked()
{
    if ( isSizeTooBig() )
    {
        maemoBanner("Widget size too big, decrease it");
        return;
    }

    saveSettings();
    emit saveButtonClicked();
    close();
}

void SettingsDialog::createCalendarsCheckboxes()
{
    QList<QOrganizerCollection> collectionsList = defaultManager->collections();

    for (int i = 0; i < collectionsList.size(); i++)
    {
        QCheckBox* checkBox = new QCheckBox;
        ui->calendarsLayout->addWidget(checkBox);

        QString name = collectionsList.at(i).metaData("Name").toString();
        if ( name == "cal_ti_calendar_private" )
        {
            name = "Private";
        }
        checkBox->setText(QString::fromUtf8(name.toStdString().c_str()));

        QString id = collectionsList.at(i).id().toString();
        checkBox->setProperty("id", id);

        if ( collectionIdsList.contains( id ) )
        {
            checkBox->setChecked(true);
        }

        QPixmap iconPixmap(30, 30);
        QColor color;

        if ( collectionsList.at(i).metaData("Color").toString() == "Dark blue" )
        {
            color = QColor("#488dff");
        }
        else if ( collectionsList.at(i).metaData("Color").toString() == "Dark green" )
        {
            color = QColor("#1aec0a");
        }
        else if ( collectionsList.at(i).metaData("Color").toString() == "Dark red" )
        {
            color = QColor("#ff54a0");
        }
        else if ( collectionsList.at(i).metaData("Color").toString() == "Orange" )
        {
            color = QColor("#e38600");
        }
        else if ( collectionsList.at(i).metaData("Color").toString() == "Violet" )
        {
            color = QColor("#d954ff");
        }
        else if ( collectionsList.at(i).metaData("Color").toString() == "Yellow" )
        {
            color = QColor("Yellow");
        }
        else if ( collectionsList.at(i).metaData("Color").toString() == "White" )
        {
            color = QColor("White");
        }
        else if ( collectionsList.at(i).metaData("Color").toString() == "Blue" )
        {
            color = QColor("Cyan");
        }
        else if ( collectionsList.at(i).metaData("Color").toString() == "Red" )
        {
            color = QColor("#ff9393");
        }
        else if ( collectionsList.at(i).metaData("Color").toString() == "Green" )
        {
            color = QColor("#b4ff00");
        }

        iconPixmap.fill( color );
        checkBox->setIcon(QIcon(iconPixmap));

        calendarsCheckBoxList.append(checkBox);
    }
}

bool SettingsDialog::isSizeTooBig()
{
    int numberOfRows = ui->numberOfRowsSpinBox->value();
    int rowHeight = ui->rowHeightSpinBox->value();

    int totalHeight = ((numberOfRows+1) * (rowHeight+1)) - 1;

    if ( totalHeight > 425 )
    {
        return true;
    }
    else
    {
        return false;
    }
}
